    # -*- coding: utf-8 -*- 
# @Time    : 03/24/2024 
# @Author  : nzooherd 
# @File    : message_service.py 
# @Software: Visual Studio Code

import base64
from datetime import datetime
from logging import getLogger
from typing import Optional, Union
from telethon.events import NewMessage, ButtonCallback
from telethon.types import Message as TelethonMessage

from nzooherd_pyzen.classes import Singleton

from dao.message_dao import MessageDao
from dao.model import Message
from template import BaseTemplate

logger = getLogger()

class MessageService(metaclass=Singleton):

    SELFREF_PLACEHOLDER = "%SELFREF_PLACEHOLDER%"

    def __init__(self) -> None:
        self.message_dao = MessageDao()

    
    async def placeholder_message(self) -> Message:
        message = Message(message_id=0, from_id="0", send_id="0", content="0", timestamp=datetime.now())
        return await self.message_dao.insert(message)
    
    async def get_by_telegram_message_id(self, message_id: int) -> Optional[Message]:
        return await self.message_dao.select_by_telegram_id(message_id)

    async def new_message(self, message: Message, t_message: TelethonMessage, template: BaseTemplate):
        message.message_id = t_message.id
        message.from_id = "0" 
        message.send_id = str(t_message.chat.id)
        message.content = template.content()
        message.button = template.button()
        message.timestamp = t_message.date
        await self.message_dao.update_by_id(message)
   
    async def edit_message(self, message: Message, edit_message_id: int, t_message: TelethonMessage, template: BaseTemplate):
        message.message_id = t_message.id
        message.edit_message_id = edit_message_id
        message.from_id = "0" 
        message.send_id = str(t_message.chat.id)
        message.content = template.content()
        message.button = template.button()
        message.timestamp = t_message.date
        await self.message_dao.update_by_id(message)
 
    async def receive_message(self, update: Union[NewMessage, ButtonCallback]):
        """

        """
        if isinstance(update, NewMessage):
            from_id, send_id = update.chat.id, 0
            content = update.text
            timestamp = update.date
            id = update.id

        elif isinstance(update, ButtonCallback):
            from_id, send_id = update._raw.user_id, 0
            content = base64.b64encode(update._raw.data).decode("utf-8")
            timestamp = datetime.now()
            #TODO()
            id = int(str(update._raw.query_id)[0:8])

        message = Message(message_id=id, from_id=str(from_id), send_id=str(send_id), 
                              content=content, timestamp=timestamp)

        await self.message_dao.insert(message)
    
    async def select_by_id(self, message_id: int) -> Optional[Message]:
        """
        """
        return await self.message_dao.select_by_id(message_id)

